import pygame
import math
from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.GLU import *
class InputHandler:
    car_rotation=0
    car_x=0
    car_y=0
    camera_rotation=0
    camera_zoom=1.0
    selected=0
    old_selected=0
    def check_selected():
        if InputHandler.selected==InputHandler.old_selected:
            return False
        else:
            InputHandler.old_selected=InputHandler.selected
            return True
    def handle_keys():
        """
        Handles keyboard input to control the car's movement and rotation.
        """
        
        keys = pygame.key.get_pressed()
        if keys[pygame.K_w]:
            InputHandler.car_x += 0.1 * math.cos(math.radians(InputHandler.car_rotation))
            InputHandler.car_y += 0.1 * math.sin(math.radians(InputHandler.car_rotation))
        if keys[pygame.K_s]:
            InputHandler.car_x -= 0.1 * math.cos(math.radians(InputHandler.car_rotation))
            InputHandler.car_y -= 0.1 * math.sin(math.radians(InputHandler.car_rotation))
        if keys[pygame.K_a]:
            InputHandler.car_rotation += 1.0
        if keys[pygame.K_d]:
            InputHandler.car_rotation -= 1.0
        if keys[pygame.K_q]:
        
            pygame.quit()
            sys.exit()
        if keys[pygame.K_1]:
            InputHandler.selected=0
            #print(InputHandler.selected)
        if keys[pygame.K_2]:
            InputHandler.selected=1
            #print(InputHandler.selected)
        if keys[pygame.K_3]:
            InputHandler.selected=2
            #print(InputHandler.selected)
            pass
        if keys[pygame.K_4]:
            
            InputHandler.selected=3
            #print(InputHandler.selected)
            pass
        if keys[pygame.K_5]:
            
            selected=4
            #print(InputHandler.selected)
            pass
        if keys[pygame.K_6]:
            
            selected=5
            #print(InputHandler.selected)
            pass
        if keys[pygame.K_7]:
            
            selected=6
            #print(InputHandler.selected)
            pass
        if keys[pygame.K_8]:
            
            selected=7
            #print(InputHandler.selected)
            pass
        if keys[pygame.K_9]:
            
            selected=8
            #print(InputHandler.selected)
            pass
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
    def handle_mouse(event):
        """
        Handles mouse input to control the camera's rotation and zoom.
        """
        if event.type == pygame.QUIT:
                pygame.quit()
                return
        if event.type == pygame.MOUSEBUTTONDOWN:
            
            if event.button == 4:
                # Scroll up, zoom in
                InputHandler.camera_zoom *= 1.1
            elif event.button == 5:
                # Scroll down, zoom out
                InputHandler.camera_zoom *= 0.9
        elif event.type == pygame.MOUSEMOTION:
            if event.buttons[0] == 1:
                # Left button pressed, rotate
                InputHandler.camera_rotation += event.rel[0] * 0.01


import os
from taisim2.logging_system import logger
VERSION='0.1'
logger.info(f"VERSION : \033[92m{VERSION}\033[0m",)

LEVEL1 = os.path.join(os.path.dirname(__file__), 'data/LineFollower1.png')
LEVEL2 = os.path.join(os.path.dirname(__file__), 'data/LineFollower2.png')
LEVEL3 = os.path.join(os.path.dirname(__file__), 'data/LaneKeeper1.png')
LEVEL4 = os.path.join(os.path.dirname(__file__), 'data/LaneKeeper2.png')
LEVEL5 = os.path.join(os.path.dirname(__file__), 'data/LaneKeeper3.png')
LEVEL6 = os.path.join(os.path.dirname(__file__), 'data/Cropps1.png')
LEVEL7 = os.path.join(os.path.dirname(__file__), 'data/Cropps2.png')
LOGO   = os.path.join(os.path.dirname(__file__), 'data/taisim_logo.png')

data=[LEVEL1,LEVEL2,LEVEL3,LEVEL4,LEVEL5,LEVEL6,LEVEL7,LOGO]
ok=0
for item in data:

    if os.path.exists(item) and ok==0:
        continue
    else:
        ok=1
        break

    
if ok==1:
    logger.info("ASSETS : \033[92m Not OK\033[0m")
else:
    logger.info("ASSETS : \033[92m OK\033[0m")

SIMULATOR=os.path.join(os.path.dirname(__file__), 'simulator.py')

UTILS=os.path.join(os.path.dirname(__file__), 'utils.py')
if os.path.exists(SIMULATOR):
    logger.info("SCRIPTS : \033[92mOK\033[0m")



def rgb_to_ansi_escape(red, green, blue):
    # Convert normalized values to standard 8-bit integers (0-255)
    r_int = round(red * 255)
    g_int = round(green * 255)
    b_int = round(blue * 255)

    # Construct the ANSI escape sequence for setting text color
    ansi_color_code = f"\033[38;2;{r_int};{g_int};{b_int}m"

    return ansi_color_code
def rgb_to_ansi_background(red, green, blue):
    # Convert normalized values to standard 8-bit integers (0-255)
    r_int = round(red * 255)
    g_int = round(green * 255)
    b_int = round(blue * 255)

    # Construct the ANSI escape sequence for setting background color
    ansi_color_code = f"\033[48;2;{r_int};{g_int};{b_int}m"

    return ansi_color_code



import pygame
from taisim2.input_handler import InputHandler
from pygame import image
from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.GLU import *
from taisim2.logging_system import logger
import cv2
import numpy as np
import math
from taisim2.robot import Robot,Sensors
from taisim2.utils import rgb_to_ansi_escape, rgb_to_ansi_background,LEVEL1,LEVEL2,LEVEL3,LEVEL4,LEVEL5,LEVEL6,LEVEL7
WINDOW_WIDTH = 1000
WINDOW_HEIGHT = 600
# Texture variables
texture_id = None
# Camera variables
camera_distance = 10.0
# Ground dimensions
GROUND_SIZE = 10
# Car dimensions
CAR_SIZE = 0.2

class Simulator:
    render_id=0
    init=False
    pygame.init()
    logger.info("SIMULATOR : \033[92mSTART\033[0m")
    screen=pygame.display.set_mode((WINDOW_WIDTH, WINDOW_HEIGHT),pygame.OPENGL)#|HIDDEN)
    pygame.display.set_caption("Window 1")
    screen.fill((255,0,0))
    #pygame.event.set_grab(True)
    #slider = HorizontalSlider(screen,window_width=WINDOW_WIDTH,window_height=WINDOW_HEIGHT,slider_width=250,slider_height=20,slider_x=200,slider_y=500,slider_color=(100, 100, 100),
    #handle_color=(255, 0, 0),text_color=(0, 0, 0),max_positions=9,slider_value=0.5)
    glEnable(GL_DEPTH_TEST)
    glMatrixMode(GL_PROJECTION)
    gluPerspective(45.0, WINDOW_WIDTH / WINDOW_HEIGHT, 0.1, 100.0)
    clock = pygame.time.Clock()
    
    def isRunning():
        
        
        if not Simulator.init:
            print(Sensors.items)
            arch_color=rgb_to_ansi_background(0.8,0.8,0)
            print(f"<{arch_color}SIMUALATION ARCHITECTURE\033[0m>")
            Simulator.init=True
            if len(Robot.name):
                for j in range(len(Robot.name)):
                    
                    sensor_color=rgb_to_ansi_background(0.0,0.3,1)
                    color=rgb_to_ansi_background(Robot.r[j],Robot.g[j],Robot.b[j])
                    print(f"        \033[33m|")
                    print(f"        \033[33m|")
                    print(f"        \033[33m\u251C---\u252C\033[0m<{color}{Robot.name[j]}\033[0m> ")
                    
                    for i in range(len(Robot.robot_sensors)):
                        if Robot.name[j]==Robot.robot_sensors[i][0]:
                            
                                print("\033[33m        |   |")
                                print(f"\033[33m        |   \u251C--------\033[0m<{sensor_color}%s\033[0m> \033[92m%s\033[0m"%(Robot.robot_sensors[i][1],Robot.robot_sensors[i][2]))
                                
                color=rgb_to_ansi_background(Robot.r[InputHandler.selected],Robot.g[InputHandler.selected],Robot.b[InputHandler.selected])
                print(f"\033[0m<{color}{Robot.name[InputHandler.selected]}\033[0m> SELECtED for Remote Controll")
        for event in pygame.event.get():
            InputHandler.handle_mouse(event)     
            #button.handle_event(event)
        #Simulator.slider.draw_slider()
        InputHandler.handle_keys()
        Simulator.render_id=0  
        Simulator.clock.tick(60)
        return True
    def track(path):
        """
        Load the image file and create a texture from it.
        """
        global texture_id
        
        texture_surface = image.load(path)  # Replace 'ground_texture.jpg' with your own image file
        logger.info("TRACK SET : \033[92m%s\033[0m",path)
        texture_data = pygame.image.tostring(texture_surface, "RGB", 3)

        texture_id = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D, texture_id)
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, texture_surface.get_width(), texture_surface.get_height(),
                    0, GL_RGB, GL_UNSIGNED_BYTE, texture_data)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
    def render():
        """
        Renders the scene using OpenGL.
        """
        
        

        glViewport(0, 0, WINDOW_WIDTH , WINDOW_HEIGHT)  # Left half of the screen
        glScissor(0, 0, WINDOW_WIDTH , WINDOW_HEIGHT)  # Set scissor rectangle
        glEnable(GL_SCISSOR_TEST)  # Enable scissor test
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        if(InputHandler.check_selected()):
            InputHandler.car_x=Robot.x[InputHandler.selected]
            InputHandler.car_y=Robot.y[InputHandler.selected]
            InputHandler.car_rotation=Robot.rotation[InputHandler.selected]
            color=rgb_to_ansi_background(Robot.r[InputHandler.selected],Robot.g[InputHandler.selected],Robot.b[InputHandler.selected])
            print(f"\033[0m<{color}{Robot.name[InputHandler.selected]}\033[0m> SELECTED for remote control")
        gluLookAt(
            camera_distance * math.cos(InputHandler.camera_rotation), -camera_distance * math.sin(InputHandler.camera_rotation), 3.0 * InputHandler.camera_zoom,
            InputHandler.car_x, InputHandler.car_y, 0.0,
            0.0, 0.0, 1.0
        )
        draw_ground()
        
        #robot=Robot(name="hello",robot_type="drone",x=InputHandler.car_x,y=InputHandler.car_y,size=0.2,rotation=InputHandler.car_rotation)
        if Robot.count:
            for i in range(Robot.count):
                
                
                if i==InputHandler.selected:
                    Robot.x[i]=InputHandler.car_x
                    Robot.y[i]=InputHandler.car_y
                
                    Robot.rotation[i]=InputHandler.car_rotation
                glPushMatrix()
                draw_cuboid(i,1)
                glPopMatrix()
                Robot.draw_text(Robot.name[i],Robot.x[i],Robot.y[i],1,0.001,rect_color=(Robot.r[i],Robot.g[i],Robot.b[i],0.5))
            for i in range(len(Sensors.items)):
                glPushMatrix()
                draw_camera(Sensors.items[i][0],Sensors.items[i][1],Sensors.items[i][2],Sensors.items[i][3],Sensors.items[i][4],Sensors.items[i][5],Sensors.items[i][6])
                glPopMatrix()
        
    
        
        
        pixels = glReadPixels(0, 0, WINDOW_WIDTH, WINDOW_HEIGHT, GL_RGB, GL_UNSIGNED_BYTE)
        #glClearColor(0.0, 0.0, 0.0, 1.0)  # Set clear color to black
        #glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glDisable(GL_SCISSOR_TEST)  # Disable scissor test after rendering
        image_data = np.frombuffer(pixels, dtype=np.uint8).reshape(WINDOW_HEIGHT, WINDOW_WIDTH, 3)
        image_data = cv2.flip(image_data, 0)  # Flip the image due to OpenGL's coordinate system
        
        glFlush()
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        glEnable(GL_DEPTH_TEST)
        
        #pygame.display.flip()
        return image_data


def draw_camera(x,y,z,i,xy,yz,zx):
    cuboid_vertices = [
    [-0.03, -0.03, 0.02],    # Bottom left near
    [0.03, -0.03, 0.02],     # Bottom right near
    [0.03, 0.03, 0.02],      # Top right near
    [-0.03, 0.03, 0.02],     # Top left near
    [-0.03, -0.03, 0.05],  # Bottom left far
    [0.03, -0.03, 0.05],   # Bottom right far
    [0.03, 0.03, 0.05],    # Top right far
    [-0.03, 0.03, 0.05]    # Top left far
]
    pyramid_vertices = [
        
    [0.0, 0.0, 0.1],   # Apex of the pyramid
    [0.2, 0.2, 0.5],
    [-0.2, 0.2, 0.5],
    [-0.2, -0.2, 0.5],  # Base vertices
    
    
    [0.2, -0.2, 0.5]
]

# Define the cuboid faces
    cuboid_faces = [
    [0, 1, 2, 3],  # Bottom face
    [4, 5, 6, 7],  # Top face
    [0, 4, 5, 1],  # Near face
    [1, 5, 6, 2],  # Right face
    [2, 6, 7, 3],  # Far face
    [3, 7, 4, 0]   # Left face
]
    glTranslatef(+x+Robot.x[i], +y+Robot.y[i], +z+Robot.z[i])
    glRotatef(Robot.rotation[i]+xy, 0.0, 0.0, 1.0)  # Rotate around z-axis (upward direction)
    glRotatef(yz, 1.0, 0.0, 0.0)
    glRotatef(zx,0,1,0)
    glRotatef(90, 0.0, 1.0, 0.0)  # Rotate around z-axis (upward direction)
    glBegin(GL_QUADS)
    
    glColor3f(1, 1, 1)  # Magenta color for the car
    for face in cuboid_faces:
        
        for vertex_id in face:
            glVertex3fv(cuboid_vertices[vertex_id])
    glEnd()
   
    glColor3f(0.0, 0.0, 0.0)  # Set color to black
   
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)  # Set polygon mode to draw lines
    glLineWidth(4.0)  # Set line width if desired

    for face in cuboid_faces:
        glBegin(GL_LINE_LOOP)
        for vertex_id in face:
            glVertex3fv(cuboid_vertices[vertex_id])
        glEnd()
    glBegin(GL_QUADS)
    colors=[[1.0, 1.0, 1.0],[1,0,0],[0,1,0],[0,0,1],[1,1,0]]  # Set color to black
    for surface in [
        [1, 2, 3, 4],  # Base
        [0, 1, 2],     # Triangle 1
        [0, 2, 3],     # Triangle 2
        [0, 3, 4],     # Triangle 3
        [0, 4, 1]      # Triangle 4
    ]:
        for i,vertex in enumerate(surface):
            glColor3f(colors[i][0],colors[i][1],colors[i][2]) 
            glVertex3fv(pyramid_vertices[vertex])
    glEnd()
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)  # Reset to fill mode
    
    glColor3f(Robot.r[i], Robot.b[i], Robot.g[i])  # Reset to original color
def draw_ground():
    """
    Draws the ground with the applied texture.
    """
    glEnable(GL_TEXTURE_2D)
    glBindTexture(GL_TEXTURE_2D, texture_id)
    glColor3f(1.0, 1.0, 1.0)
    glBegin(GL_QUADS)
    glTexCoord2f(0.0, 0.0)
    glVertex3f(-GROUND_SIZE / 2, -GROUND_SIZE / 2, 0.0)  # Bottom left
    glTexCoord2f(1.0, 0.0)
    glVertex3f(GROUND_SIZE / 2, -GROUND_SIZE / 2, 0.0)  # Bottom right
    glTexCoord2f(1.0, 1.0)
    glVertex3f(GROUND_SIZE / 2, GROUND_SIZE / 2, 0.0)  # Top right
    glTexCoord2f(0.0, 1.0)
    glVertex3f(-GROUND_SIZE / 2, GROUND_SIZE / 2, 0.0)  # Top left
    glEnd()
    glDisable(GL_TEXTURE_2D)


def draw_cuboid(i,ok):
    cuboid_vertices = [
    [-Robot.size[i], -Robot.size[i]/2, 0.02],    # Bottom left near
    [Robot.size[i], -Robot.size[i]/2, 0.02],     # Bottom right near
    [Robot.size[i], Robot.size[i]/2, 0.02],      # Top right near
    [-Robot.size[i], Robot.size[i]/2, 0.02],     # Top left near
    [-Robot.size[i], -Robot.size[i]/2, 0.05],  # Bottom left far
    [Robot.size[i], -Robot.size[i]/2, 0.05],   # Bottom right far
    [Robot.size[i], Robot.size[i]/2, 0.05],    # Top right far
    [-Robot.size[i], Robot.size[i]/2, 0.05]    # Top left far
]

# Define the cuboid faces
    cuboid_faces = [
    [0, 1, 2, 3],  # Bottom face
    [4, 5, 6, 7],  # Top face
    [0, 4, 5, 1],  # Near face
    [1, 5, 6, 2],  # Right face
    [2, 6, 7, 3],  # Far face
    [3, 7, 4, 0]   # Left face
]
    glTranslatef(Robot.x[i], Robot.y[i], Robot.z[i])
    glRotatef(Robot.rotation[i], 0.0, 0.0, 1.0)  # Rotate around z-axis (upward direction)
    glBegin(GL_QUADS)
    
    glColor3f(Robot.r[i], Robot.g[i], Robot.b[i])  # Magenta color for the car
    for face in cuboid_faces:
        
        for vertex_id in face:
            glVertex3fv(cuboid_vertices[vertex_id])
    glEnd()
    mean =(Robot.r[i]+Robot.b[i]+Robot.g[i])/3
    if (mean)>=0.3:
        glColor3f(0.0, 0.0, 0.0)  # Set color to black
    else:
        glColor3f(1.0, 1.0, 1.0)  # Set color to white
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)  # Set polygon mode to draw lines
    glLineWidth(4.0)  # Set line width if desired

    for face in cuboid_faces:
        glBegin(GL_LINE_LOOP)
        for vertex_id in face:
            glVertex3fv(cuboid_vertices[vertex_id])
        glEnd()

    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)  # Reset to fill mode
    if ok==1:
        glColor3f(Robot.r[i], Robot.g[i], Robot.b[i])  # Reset to original color
    else:
        glColor3f(Robot.r[i], Robot.b[i], Robot.g[i])  # Reset to original color




"""
info:This module contains the implememntation of the SparkVerse Simulator
autor: Tucudean Adrian-Ionut
date: 25.11.2023
email: Tucudean.Adrian.Ionut@outlook.com
license: MIT
"""

import os
from setuptools import setup, find_packages
#from src.tucu.external_data import VERSION
here = os.path.abspath(os.path.dirname(__file__))
#README = open(os.path.join(here, 'README-pypi.rst')).read()



install_requires = [
    # List your project dependencies here.
    # For more details, see:
    # http://packages.python.org/distribute/setuptools.html#declaring-dependencies
    'requests'	
]


setup(name='taisim2',
    version='0.1',
    description="Simulation Library for Multi-Robot Systems(this one works",
    long_description="Simulation Library for Multi-Robot Systems",
    keywords='Simulator computer vision Advanced Driving',
    author='Tucudean Adrian-Ionut',
    author_email='Tucudean.Adrian.Ionut@outlook.com',
    url='https://github.com/Amporu/Taisim2',
    license='MIT',
    packages=find_packages('src','src/taisim2/data'),
    package_dir = {'': 'src'},include_package_data=True,
    package_data={'taisim2':['data/*.png',"*.py"]},
    zip_safe=False,
    install_requires=['opencv-python','pygame','pyOpenGL'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    entry_points={
    'console_scripts': [
        'taisim2 = taisim2:main',
    ],
},
)


from taisim2.simulator import Simulator, WINDOW_HEIGHT,WINDOW_WIDTH, CAR_SIZE,draw_ground
from taisim2.simulator import draw_cuboid
from taisim2.input_handler import InputHandler
from taisim2.robot import Robot,Sensors
import cv2
import math
import numpy as np

from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.GLU import *


class Camera:
    def __init__(self,robot,tag,pos_x=0,pos_y=0,pos_z=0.1,frame_width=640,frame_height=480,format="BGR",rotationXY=0,rotationYZ=0,rotationZX=0,fov=45,near_clip=0.1,far_clip=100):
        self.type="CAMERA"
        self.nickname=tag
        self.robot=robot
        self.x=pos_x
        self.y=pos_y
        self.z=pos_z
        self.frame_width=frame_width
        self.frame_height=frame_height
        self.format=format
        self.rotationXY=rotationXY
        self.rotationYZ=rotationYZ
        self.rotationZX=rotationZX
        self.fov=fov
        self.near_clip=near_clip
        self.far_clip=far_clip
        Robot.robot_sensors.append([self.robot.name,self.type,tag,self.robot.robot_type])
        #x y z rx ry rz ,yaw,pitch,roll or whatever, I dont know english that good
        Sensors.items.append([self.x,self.y,self.z,self.robot.count-1,self.rotationXY,rotationYZ,rotationZX])
        
       #print(Robot.robot_sensors)
    
    
    def read(self,depth=False):
        Simulator.render_id=Simulator.render_id+1
        id=Simulator.render_id
        # Create variables for FBO size
        fbo_width = self.frame_width  # Define FBO width
        fbo_height = self.frame_height  # Define FBO height

        # Create and bind a Framebuffer Object (FBO)
        fbo = glGenFramebuffers(1)
        glBindFramebuffer(GL_FRAMEBUFFER, fbo)
        
        # Create a texture for off-screen rendering
        texture = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D, texture)
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, fbo_width, fbo_height, 0, GL_RGB, GL_UNSIGNED_BYTE, None)
        glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, texture, 0)
        depth_buffer = glGenRenderbuffers(1)
        glBindRenderbuffer(GL_RENDERBUFFER, depth_buffer)
        glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, fbo_width, fbo_height)
        glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, depth_buffer)
        glEnable(GL_DEPTH_TEST)

        # Check if FBO is complete
        if glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE:
            print("Framebuffer is not complete!")
            glBindFramebuffer(GL_FRAMEBUFFER, 0)
            return None
        gluPerspective(self.fov, fbo_width / fbo_height, self.near_clip, self.far_clip)
        # Set the viewport and render to the FBO
        glViewport(0, 0, fbo_width, fbo_height)
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()
        
        # Render your scene here using the desired viewport and rendering code

        # Read pixels from the FBO
        glBindFramebuffer(GL_FRAMEBUFFER, fbo)
        camera_x = Robot.x[self.robot.count-1]+self.x + CAR_SIZE * math.cos(math.radians(Robot.rotation[self.robot.count-1]))
        camera_y = Robot.y[self.robot.count-1]+self.y + CAR_SIZE * math.sin(math.radians(Robot.rotation[self.robot.count-1]))
        camera_look_x = camera_x + math.cos(math.radians(Robot.rotation[self.robot.count-1]))
        camera_look_y = camera_y + math.sin(math.radians(Robot.rotation[self.robot.count-1]))
        gluLookAt(
            camera_x, camera_y,0.1+self.z+Robot.z[self.robot.count-1],  # Camera position (slightly above the car)
            camera_look_x, camera_look_y,0.1+self.z+Robot.z[self.robot.count-1],  # Look-at point (in the car's direction)
            0.0, 0.0, 1.0
        )
        
        
        draw_ground()
        if Robot.count:
            for i in range(Robot.count):
                if i==InputHandler.selected:
        
                    Robot.x[i]=InputHandler.car_x
                    Robot.y[i]=InputHandler.car_y
                
                    Robot.rotation[i]=InputHandler.car_rotation
                
                glPushMatrix()
                
                draw_cuboid(i,0)
                glPopMatrix()
        glDisable(GL_DEPTH_TEST)
        
        if depth==True:
            
            depth_pixels = glReadPixels(0, 0, fbo_width, fbo_height, GL_DEPTH_COMPONENT, GL_FLOAT)
            depth_data = np.frombuffer(depth_pixels, dtype=np.float32).reshape(fbo_height, fbo_width)
        
            depth_data = cv2.flip(depth_data, 0)  # 0 for vertical flip
                    
        pixels = glReadPixels(0, 0, fbo_width, fbo_height, GL_BGR, GL_UNSIGNED_BYTE)

        # Unbind FBO and clean up
        glBindFramebuffer(GL_FRAMEBUFFER, 0)
        glDeleteFramebuffers(1, [fbo])
        glDeleteTextures(1, [texture])
        glBindRenderbuffer(GL_RENDERBUFFER,0)
        glDeleteRenderbuffers(1,[depth_buffer])

        # Process the pixels (if needed) and return the image data
        #print(id)
        image_data = np.frombuffer(pixels, dtype=np.uint8).reshape(fbo_height, fbo_width, 3)
        
        image_data = cv2.flip(image_data, 0)  # 0 for vertical flip
        (w, h), _ = cv2.getTextSize("Robot: "+str(Robot.name[self.robot.count-1]), cv2.FONT_HERSHEY_SIMPLEX, 0.3, 1)
        cv2.rectangle(img=image_data,pt1=(10,10),pt2=(10+w,20+h//2),color=(int(Robot.b[self.robot.count-1]*255),int(Robot.g[self.robot.count-1]*255),int(Robot.r[self.robot.count-1]*255)),thickness=-1)
        
        cv2.putText(image_data, "Robot: "+str(Robot.name[self.robot.count-1]), (10, 20),
                    cv2.FONT_HERSHEY_SIMPLEX, 0.3, (0,0,0), 1)
        (w, h), _ = cv2.getTextSize("TAG: "+str(self.nickname), cv2.FONT_HERSHEY_SIMPLEX, 0.3, 1)
        cv2.rectangle(img=image_data,pt1=(10,30),pt2=(10+w,40+h//2),color=(0,255,0),thickness=-1)
        cv2.putText(image_data, "TAG: "+str(self.nickname), (10, 40),
                    cv2.FONT_HERSHEY_SIMPLEX, 0.3, (0,0,0), 1)
        #image_data= cv2.rectangle(img=image_data,pt1=(0,0),pt2=(100,400),color=(255,0,0),thickness=-1)
        if self.format=="RGB":
            image_data=cv2.cvtColor(image_data,cv2.COLOR_BGR2RGB)
        if self.format=="GRAY":
            image_data=cv2.cvtColor(image_data,cv2.COLOR_BGR2GRAY)
        if depth==False:
            return image_data
        if depth==True:
            return image_data,depth_data



class GPS:
    def __init__(self,robot,name):
        self.robot=robot
        self.x=robot.x
        self.y=robot.y
        
        self.type="GPS"

        Robot.robot_sensors.append([self.robot.name,self.type,name,self.robot.robot_type])
    def read(self):
        return Robot.x[self.robot.count-1],Robot.y[self.robot.count-1],Robot.z[self.robot.count-1]
class COMPASS:
    def __init__(self,robot,name):
        self.robot=robot
        self.rotation=robot.rotation
        
        self.type="COMPASS"

        Robot.robot_sensors.append([self.robot.name,self.type,name,self.robot.robot_type])
    def read(self):
        return abs(Robot.rotation[self.robot.count-1]%360)



from OpenGL.GL import *
from OpenGL.GLUT import *
from taisim2.input_handler import InputHandler
import numpy as np
import math
import random
class Sensors:
    items=[]
class Robot:
    x=[]
    y=[]
    z=[]
    rotation=[]
    name=[]
    size=[]
    count=0
    robot_sensors=[]
    r=[]
    g=[]
    b=[]
    
    
    type=[]
    def __init__(self,tag,x=0,y=0,z=0,size=0.3,rotation=0):
        

        Robot.r.append(random.random())
        Robot.g.append(random.random())
        Robot.b.append(random.random())
        self.robot_id=Robot.count
        
        Robot.count=Robot.count+1
        Robot.type.append("robot")
        Robot.name.append(tag)
        Robot.x.append(x)
        Robot.y.append(y)
        Robot.z.append(z)
        Robot.size.append(size)
        Robot.rotation.append(rotation)
        self.x=x
        self.y=y
        self.z=z
        self.name=tag
        self.robot_type="robot"
        self.count=Robot.count
        self.rotation=rotation
        
        
        glPushMatrix()
        glTranslatef(self.x, self.y, self.z)
        glRotatef(Robot.rotation[self.count-1], 0.0, 0.0, 1.0)  # Rotate around z-axis (upward direction)
        glBegin(GL_QUADS)
        glColor3f(1.0, 0, 1.0)  # Magenta color for the car
        glVertex3f(-Robot.size[self.count-1], -Robot.size[self.count-1], 0.1)  # Bottom left
        glVertex3f(Robot.size[self.count-1], -Robot.size[self.count-1], 0.1)  # Bottom right
        glVertex3f(Robot.size[self.count-1], Robot.size[self.count-1], 0.1)  # Top right
        glVertex3f(-Robot.size[self.count-1], Robot.size[self.count-1], 0.1)  # Top left
        
        glEnd()
    
       
        
        """
        Draws the car using OpenGL.
        """
        
        glPopMatrix()
        Robot.draw_text(self.name,self.x,self.y,1,0.001)
    def move(self,linear_velocity=0,angular_velocity=0,altitude=0):
        Robot.rotation[self.count-1]+=angular_velocity
        Robot.x[self.count-1]+=linear_velocity*math.cos(math.radians(Robot.rotation[self.count-1]))
        Robot.y[self.count-1]+= linear_velocity * math.sin(math.radians(Robot.rotation[self.count-1]))
        Robot.z[self.count-1]=altitude
        pass
    def draw_text(text, x, y, z, size, rect_color=(0.0, 0.0, 1.0,0.5)):
        # Calculate the dimensions of the text in pixels (you may need to adjust this)
        text_width_pixels = len(text) * 0.1
        text_height_pixels = 1

        # Calculate the vertices for the rectangle
        rect_left = 0
        rect_right = len(text) * 0.5*(InputHandler.camera_zoom)
        rect_bottom = -text_height_pixels /8
        rect_top = text_height_pixels /2*InputHandler.camera_zoom
        camera_distance=10
        # Calculate the direction vector from camera to (0, 0, 1)
        camera_position = np.array([float(x), float(y), float(z)])
        target_position = np.array([camera_distance * math.cos(InputHandler.camera_rotation), -camera_distance * math.sin(InputHandler.camera_rotation), 3*InputHandler.camera_zoom])
        direction = target_position - camera_position
        direction /= np.linalg.norm(direction)

        # Calculate the rotation matrix to align text and background with the camera
        up_vector = np.array([0.0, 0.0, 1.0])
        right_vector = np.cross(up_vector, direction)
        up_vector = np.cross(direction, right_vector)
        rotation_matrix = np.column_stack((right_vector, up_vector, direction, [0.0, 0.0, 0.0]))
        rotation_matrix = np.vstack((rotation_matrix, [0.0, 0.0, 0.0, 1.0]))

        glPushMatrix()
        glTranslatef(x, y, z)

        # Apply the rotation matrix to face the camera
        glMultMatrixf(rotation_matrix.T)

        # Draw the rectangle as the background
        glColor4f(*rect_color)
        glBegin(GL_QUADS)
        glVertex3f(rect_left, rect_bottom,-1)
        glVertex3f(5*rect_right//6, rect_bottom,-1)
        glVertex3f(5*rect_right//6, rect_top,-1)
        glVertex3f(rect_left, rect_top,-1)
        glEnd()

        # Scale the background and text
        glScalef(size*InputHandler.camera_zoom*4, size*InputHandler.camera_zoom*4, size*InputHandler.camera_zoom*4)

        # Use the shader program
        glColor3f(1.0, 1.0, 1.0)  # Set the text color (white in this example)
        for char in text:
            glutStrokeCharacter(GLUT_STROKE_ROMAN, ord(char))

        glPopMatrix()



import logging

# Create a logger
logger = logging.getLogger('SIMULATOR')
logger.setLevel(logging.DEBUG)

# Create a console handler
console_handler = logging.StreamHandler()

# Create a custom formatter with color codes
class ColoredFormatter(logging.Formatter):
    """color configuration"""
    def format(self, record):
        # Define color codes for level names
        color_codes = {
            logging.DEBUG: '\033[94m',     # Blue
            logging.INFO: '\033[92m',      # Green
            logging.WARNING: '\033[93m',   # Yellow
            logging.ERROR: '\033[91m',     # Red
            logging.CRITICAL: '\033[91m'   # Red
        }

        # Define color reset code
        color_reset = '\033[0m'

        # Apply color to level name
        levelname = record.levelname
        name=record.name
        colored_levelname = f"{color_codes[record.levelno]}{levelname}{color_reset}"
        colored_name=f"{color_codes[logging.DEBUG]}{name}{color_reset}"
        record.levelname = colored_levelname
        record.name=colored_name

        # Format the log message
        message = super().format(record)

        return message

# Create a colored formatter
formatter = ColoredFormatter('[%(levelname)s] <%(name)s> %(message)s')

# Set the formatter for the console handler
console_handler.setFormatter(formatter)

# Add the console handler to the logger
logger.addHandler(console_handler)